class CodeGenerationException(Exception):
    """Exception raised when code generation fails."""
    
    def __init__(self, message : str):
        self.message = message
        super().__init__(self.message)

class FunctionRuntimeException(Exception):
    """Exception raised when a function fails."""
    
    def __init__(self, message : str):
        self.message = message
        super().__init__(self.message)

class ParamsException(Exception):
    """Exception raised when params are mismatch."""
    
    def __init__(self, message : str):
        self.message = message
        super().__init__(self.message)



from collections import defaultdict

class Graph:
    def __init__(self, vertices):
        self.graph = defaultdict(list) 
        self.V = vertices 

    def addEdge(self, u, v):
        self.graph[u].append(v)
 
    def topologicalSortUtil(self, v, visited, stack):
        visited[v] = True
        for i in self.graph[v]:
            if visited[i] == False:
                self.topologicalSortUtil(i, visited, stack)
        stack.append(v)

    def topologicalSort(self):
        visited = [False]*self.V
        stack = []
        for i in range(self.V):
            if visited[i] == False:
                self.topologicalSortUtil(i, visited, stack)
        return(stack[::-1]) 


import pandas as pd
import requests
from pydantic import BaseModel
import woodwork as ww

class Dataset(BaseModel):
  url :str
  dataframe : pd.DataFrame

  def __init__(self,url) -> None:
    self.url = url
    self.dataframe = self.__load_dataset(url)

  def __load_dataset(self,url : str) -> pd.DataFrame:
      local_filename = url.split('/')[-1]
      with requests.get(url, stream=True) as r:
          r.raise_for_status()
          with open(local_filename, "wb") as f:
              for chunk in r.iter_content(chunk_size=8192):
                  f.write(chunk)
      return pd.read_csv(local_filename)
  
  def parse_dataset(self):
    self.dataframe.ww.init() 
    logical_types = self.dataframe.ww.logical_types
    logical_types_dict = {}
    for col in logical_types:
        logical_types_dict[col] = logical_types[col]
    return logical_types_dict



class EssentialsHandler:

    def __init__(self) -> None:
        pass
        
    def return_function(self, function_name: str,params:dict):
        essentials = {
            "Train-Test Split": self.__train_test_split,
            "Train Model": self.__train_model,
        }
        return essentials[function_name](params)
 
    def __train_test_split(self, params):
        parameters = {
            "test_size": params["test_size"],
        }
        return f"training_dataset, test_dataset = huble.sklearn.train_test_split(data=data,parameters={parameters})"


    def __train_model(self, params):
        return f"Model = huble.sklearn.train_model(data=training_dataset, model=model, column='{params['target_column']}')"




from huble.sklearn.metrics import log_metrics

def evaluate_model(model, test_dataset, target_column, task_type):
    y_test = test_dataset[target_column]
    X_test = test_dataset.drop([target_column], axis=1)
    y_pred = model.predict(X_test)
    metrics = log_metrics(y_test, y_pred, task_type)
    #TODO : Write upload metrics
    return metrics



import re
from re import sub
from sklearn import impute
from sklearn import preprocessing
import pandas as pd
from scipy import stats
import numpy as np
from AutoClean import AutoClean
from ...error.decorators import function_error_handling

@function_error_handling("remove_nan_values")
def remove_nan_values(**params):
    data = params['data']
    data = data.dropna(**params["parameters"])
    return data


@function_error_handling("replace_nan_values")
def replace_nan_values(**params):
    imputer = impute.SimpleImputer(**params["parameters"])
    data = params['data']
    data[(params['column'])] = imputer.fit_transform(data[(params['column'])].values.reshape(-1,1))[:,0]
    return data


@function_error_handling("drop_rows_columns")
def drop_rows_columns(**params):
    data = params['data']
    data = data.drop(**params["parameters"])
    return data


@function_error_handling("remove_outliers")
def remove_outliers(**params):
    data = params['data']
    for column in params["columns"]:
        print(column)
        data = data[
            (np.abs(stats.zscore(data[(column)])) < 3)
        ]
    return data


@function_error_handling("drop_duplicates")
def drop_duplicates(**params):
    data = params['data']
    data = data.drop_duplicates(**params["parameters"])
    return data


@function_error_handling("change_data_type")
def change_data_type(**params):
    data = params['data']
    data[(params['column'])] = data[(params['column'])].astype(**params["parameters"])
    return data


@function_error_handling("round_data")
def round_data(**params):
    data = params['data']
    data = data.round(**params["parameters"])
    return data


@function_error_handling("filter_dataframe")
def filter_dataframe(**params):
    data = params['data']
    data = data.filter(**params["parameters"])
    return data


@function_error_handling("truncate_dataframe")
def truncate_dataframe(**params):
    data = params['data']
    data = data.truncate(**params["parameters"])
    return data


@function_error_handling("sort_values")
def sort_values(**params):
    data = params['data']
    data = data.sort_values(**params["parameters"])
    return data


@function_error_handling("transpose")
def transpose(**params):
    data = params['data']
    data = data.transpose()
    return data


@function_error_handling("min_max_scale")
def min_max_scale(**params):
    data = params['data']
    scaler = preprocessing.MinMaxScaler(**params["parameters"])
    data[(params['column'])] = scaler.fit_transform(data[(params['column'])].values.reshape(-1,1))[:,0]
    return data


@function_error_handling("max_abs_scale")
def max_abs_scale(**params):
    data = params['data']
    scaler = preprocessing.MaxAbsScaler(**params["parameters"])
    data[(params['column'])] = scaler.fit_transform(data[(params['column'])].values.reshape(-1,1))[:,0]
    return data


@function_error_handling("robust_scale")
def robust_scale(**params):
    data = params['data']
    scaler = preprocessing.RobustScaler(**params["parameters"])
    data[(params['column'])] = scaler.fit_transform(data[(params['column'])].values.reshape(-1,1))[:,0]
    return data


@function_error_handling("standard_scale")
def standard_scale(**params):
    data = params['data']
    scaler = preprocessing.StandardScaler(**params["parameters"])
    data[(params['column'])] = scaler.fit_transform(data[(params['column'])].values.reshape(-1,1))[:,0]
    return data


@function_error_handling("normalize")
def normalize(**params):
    data = params['data']
    for column in params['columns']:
        params["X"]=column
        data = preprocessing.normalize(**params["parameters"])
    return data


@function_error_handling("ordinal_encode")
def ordinal_encode(**params):
    data = params['data']
    data_columns = params["columns"]
    enc = preprocessing.OrdinalEncoder(**params["parameters"])
    data[data_columns] = enc.fit_transform(data[data_columns])
    return data


@function_error_handling("one_hot_encode")
def one_hot_encode(**params):
    data = params['data']
    data_columns = params["columns"]
    le = preprocessing.LabelEncoder()
    data[data_columns] = data[data_columns].apply(lambda col: le.fit_transform(col))
    enc = preprocessing.OneHotEncoder(**params["parameters"])
    array_hot_encoded = enc.fit_transform(data[data_columns])
    data_hot_encoded = pd.DataFrame(array_hot_encoded, index=data.index)
    data_other_cols = data.drop(columns=data_columns)
    data_out = pd.concat([data_hot_encoded, data_other_cols], axis=1)
    print(data_out)
    return data_out
   

@function_error_handling("remove_mismatch_data")
def remove_mismatch_data(**params):
    data = params['data']
    exceptions = params['parameters']['exceptions']
    for col in data : 
        if col in exceptions : continue
        data.reset_index(drop=True, inplace=True)
        s = [False]*len(data[col])
        for i,cell in enumerate(data[col]) : 
            try : n = int(cell)
            except : s[i] = True
        t = s.count(True)
        f = s.count(False)
        st = False
        if(t>f) : st = True
        remove = [i for i in range(len(data[col])) if s[i]!=st]
        data.drop(remove,axis=0,inplace=True)
    return data


@function_error_handling("merge")
def merge(**params):
    data = params['data']
    df = data.merge(**params['parameters'])
    return df


@function_error_handling("rename_columns")
def rename_columns(**params):
    data = params['data']
    dict = params['dict']
    axis = params['axis']
    data = data.rename(dict, axis)
    return data


@function_error_handling("select_columns")
def select_columns(**params):
    data = params['data']
    df = data.select_dtypes(**params['parameters'])
    return df


@function_error_handling("clean_column_names")
def clean_column_names(**params):
    data = params['data']
    match=r"[\]\[\,\{\}\"\:]+"
    data.rename(columns=lambda x: re.sub(match, "", str(x)))
    return data


@function_error_handling("clean_data")
def clean_data(**params):
    data = params['data']
    df = AutoClean(data)
    return df.output


@function_error_handling("clip")
def clip(**params):
    data = params['data']
    data = data.clip(**params['paramters'])
    return data


@function_error_handling("split_data")
def split_data(**params):
    data = params['data']
    df_shuffled = data.sample(frac=1)
    df_splits = np.array_split(df_shuffled, 2)
    return df_splits




from .sklearn import generate_file
from .connector import Experiment


def generate_experiment(experiment_id : str, auth_key=""):
    # TODO: Implement auth keys
    experiment = Experiment(experiment_id=experiment_id)
    generate_file(graph=experiment.graph, task_type=experiment.task_type, target_column=experiment.target_column)
    return experiment
try:
  generate_experiment("b9c36c0e-5a1c-4ce2-86cd-f146dd2a5e58")
except:
  print("erroo")


from sklearn.model_selection import train_test_split  # type: ignore
import pandas as pd
from ...error.decorators import function_error_handling

@function_error_handling("train_test_split")
def train_test_split(**params):
    data = params['data']
    train_dataset, test_dataset = train_test_split(data=data, test_size=0.33, random_state=42)
    return train_dataset, test_dataset


@function_error_handling("train_model")
def train_model(**params):
    data = params['data']
    y = data[(params['column'])]
    X = data.drop(y, axis=1)
    model = params['model']
    Model =  model.fit(X, y)
    return Model



from .functions import (
    logistic_regression,
    svm_svc,
    gaussian_naive_bayes,
    multinomial_naive_bayes,
    st_gradient_descent_classifier,
    knn,
    decision_tree,
    random_forest,
    gradient_boosting,
    lgbm,
    xgboost,
    linear_regression,
    lgbm_reg,
    xgboost_reg,
    catboost_reg,
    st_gradient_descent_reg,
    kernel_ridge_reg,
    elastic_net_reg,
    bayesian_ridge_reg,
    gradient_boosting_reg,
    svm_svr,
    mean_shift,
    kmeans,
    agglomerative,
    birch,
    spectral,
    affinity_propogation,
    optics,
    dbscan
)
from .handler import *



class ModelHandler:

    def __init__(self) -> None:
        pass
        
    def return_function(self, function_name: str,params:dict):
        models = {
            "Logistic Regression": self.__logistic_regression,
            "SVM (SVC)": self.__svm_svc,
            "Gaussian Naive Bayes": self.__gaussian_naive_bayes,
            "Multinomial Naive Bayes": self.__multinomial_naive_bayes,
            "Stochastic Gradient Descent Classifier": self.__st_gradient_descent_classifier,
            "KNN": self.__knn,
            "Decision Tree Classifier": self.__decision_tree,
            "Random Forest Classifier": self.__random_forest,
            "Gradient Boosting Classifier": self.__gradient_boosting,
            "LGBM Classifier": self.__lgbm,
            "XGBoost Classifier": self.__xgboost,
            "Linear Regression": self.__linear_regression,
            "LGBM Regressor": self.__lgbm_regressor,
            "XGBoost Regressor": self.__xgboost_regressor,
            "CatBoost Regressor": self.__catboost_regressor,
            "Stochastic Gradient Descent Regression": self.__st_gradient_descent_regressor,
            "Kernel Ridge Regression": self.__kernel_ridge,
            "Elastic Net Regression": self.__elastic_net,
            "Bayesian Ridge Regression": self.__bayesian_ridge,
            "Gradient Boosting Regression": self.__gradient_boosting_reg,
            "SVM (SVR)": self.__svm_svr,
            "Mean Shift": self.__mean_shift,
            "KMeans": self.__kmeans,
            "Agglomerative Clustering": self.__agglomerative,
            "BIRCH": self.__birch,
            "Spectral Clustering": self.__spectral,
            "Affinity Propagation": self.__affinity_propogation,
            "OPTICS": self.__optics,
            "DBSCAN": self.__dbscan,
        }
        return models[function_name](params)

    def __logistic_regression(self, params):
        parameters = {
            "penalty": params["penalty"],
            "fit_intercept": params["fit_intercept"],
            "random_state": params["random_state"],
            "solver": params["solver"],
            "max_iter": params["max_iter"],
            "multi_class": params["multi_class"],
            "tol": params["tol"],
        }
        return f"model = huble.sklearn.logistic_regression({parameters})"


    def __svm_svc(self, params):
        parameters = {
            "C" : params['C'], 
            "kernel" : params['kernel'],
            "probability" : params['probability'],
            "random_state": params["random_state"],
            "max_iter": params["max_iter"],
            "decision_function_shape": params["decision_function_shape"],
            "tol": params["tol"],
        }
        return f"model = huble.sklearn.svm_svc(parameters={parameters})"

    def __gaussian_naive_bayes(self, params):
        parameters = {
            "priors" : params['priors'],
            "var_smoothing" : params['var_smoothing'],
        }
        return f"model = huble.sklearn.gaussian_naive_bayes(parameters={parameters})"

    def __multinomial_naive_bayes(self, params):
        parameters = {
            "class_prior" : params['class_prior'],
            "alpha" : params['alpha'],
            "fit_prior" : params['fit_prior'],
        }
        return f"model = huble.sklearn.multinomial_naive_bayes(parameters={parameters})"

    def __st_gradient_descent_classifier(self, params):
        parameters = {
            "loss" : params['loss'], 
            "penalty": params["penalty"],
            "fit_intercept": params["fit_intercept"],
            "alpha" : params['alpha'],
            "max_iter": params["max_iter"],
            "tol": params["tol"],
            "random_state": params["random_state"],
            "shuffle" : params['shuffle'],
            "learning_rate" : params['learning_rate'],
            "initial_learning_rate" : params['initial_learning_rate'],
            "early_stopping" : params['early_stopping'],
            "validation_fraction" : params['validation_fraction'],
        }
        return f"model = huble.sklearn.st_gradient_descent_classifier(parameters={parameters})"


    def __knn(self, params):
        parameters = {
            "n_neighbors" : params['n_neighbors'], 
            "weights" : params['weights'], 
            "algorithm" : params['algorithm'],
            "metric" : params['metric'],
        }
        return f"model = huble.sklearn.knn(parameters={parameters})"


    def __decision_tree(self, params):
        parameters = {
            "criterion" : params['criterion'],
            "splitter" : params['splitter'],
            "max_depth" : params['max_depth'],
            "max_leaf_nodes" : params['max_leaf_nodes'],
            "random_state" : params['random_state'],
        }
        return f"model = huble.sklearn.decision_tree(parameters={parameters})"


    def __random_forest(self, params):
        parameters = {
            "criterion" : params['criterion'],
            "n_estimators" : params['n_estimators'],
            "max_depth" : params['max_depth'],
            "max_leaf_nodes" : params['max_leaf_nodes'],
            "random_state" : params['random_state'],
        }
        return f"model = huble.sklearn.random_forest(parameters={parameters})"


    def __gradient_boosting(self, params):
        parameters = {
            "criterion" : params['criterion'],
            "n_estimators" : params['n_estimators'],
            "max_depth" : params['max_depth'],
            "max_leaf_nodes" : params['max_leaf_nodes'],
            "random_state" : params['random_state'],
            "loss" : params['loss'],
            "learning_rate" : params['learning_rate'],
            "subsample" : params['subsample'],
            "tol" : params['tol'],
        }
        return f"model = huble.sklearn.gradient_boosting(parameters={parameters})"


    def __lgbm(self, params):
        parameters = {
            "boosting_type" : params['boosting_type'],
            "num_leaves" : params['num_leaves'],
            "n_estimators" : params['n_estimators'],
            "max_depth" : params['max_depth'],
            "random_state" : params['random_state'],     
            "learning_rate" : params['learning_rate'],
        }
        return f"model = huble.sklearn.lgbm(parameters={parameters})"


    def __xgboost(self, params):
        parameters = {
            "n_estimators" : params['n_estimators'],
            "max_depth" : params['max_depth'],
            "random_state" : params['random_state'],     
            "learning_rate" : params['learning_rate'],
        }
        return f"model = huble.sklearn.xgboost(parameters={parameters})"


    #Regression

    def __linear_regression(self, params):
        parameters = {
            "fit_intercept" : params['fit_intercept'],
            "normalize" : params['normalize'],
            "copy_X" : params['copy_X'],
            "positive" : params['positive'],
        }
        return f"model = huble.sklearn.linear_regression(parameters={parameters})"


    def __lgbm_regressor(self, params):
        parameters = {
            "boosting_type" : params['boosting_type'],
            "num_leaves" : params['num_leaves'],
            "n_estimators" : params['n_estimators'],
            "max_depth" : params['max_depth'],
            "random_state" : params['random_state'],     
            "learning_rate" : params['learning_rate'],
        }
        return f"model = huble.sklearn.lgbm_reg(parameters={parameters})"

    def __xgboost_regressor(self, params):
        parameters = {
            "n_estimators" : params['n_estimators'],
            "max_depth" : params['max_depth'],
            "random_state" : params['random_state'],     
            "learning_rate" : params['learning_rate'],
        }
        return f"model = huble.sklearn.xgboost_reg(parameters={parameters})"

    def __catboost_regressor(self, params):
        parameters = {
            "iterations" : params['iterations'],    
            "learning_rate" : params['learning_rate'],
        }
        return f"model = huble.sklearn.catboost_reg(parameters={parameters})"

    def __st_gradient_descent_regressor(self, params):
        parameters = {
            "loss" : params['loss'], 
            "penalty": params["penalty"],
            "fit_intercept": params["fit_intercept"],
            "alpha" : params['alpha'],
            "max_iter": params["max_iter"],
            "tol": params["tol"],
            "random_state": params["random_state"],
            "shuffle" : params['shuffle'],
            "learning_rate" : params['learning_rate'],
            "early_stopping" : params['early_stopping'],
        }
        return f"model = huble.sklearn.st_gradient_descent_reg(parameters={parameters})"

    def __kernel_ridge(self, params):
        parameters = {
            "alpha" : params['alpha'],  
        }
        return f"model = huble.sklearn.kernel_ridge_reg(parameters={parameters})"

    def __elastic_net(self, params):
        parameters = {
            "fit_intercept" : params['fit_intercept'],
            "normalize" : params['normalize'],
            "copy_X" : params['copy_X'],
            "positive" : params['positive'],
            "alpha" : params['alpha'],
            "l1_ratio" : params['l1_ratio'],
            "selection" : params['selection'],
        }
        return f"model = huble.sklearn.elastic_net_reg(parameters={parameters})"

    def __bayesian_ridge(self, params):
        parameters = {
            "n_iter" : params['n_iter'], 
        }
        return f"model = huble.sklearn.bayesian_ridge_reg(parameters={parameters})"

    def __gradient_boosting_reg(self, params):
        parameters = {
            "criterion" : params['criterion'],
            "n_estimators" : params['n_estimators'],
            "max_depth" : params['max_depth'],
            "max_leaf_nodes" : params['max_leaf_nodes'],
            "random_state" : params['random_state'],
            "loss" : params['loss'],
            "learning_rate" : params['learning_rate'],
            "subsample" : params['subsample'],
            "tol" : params['tol'],
        }
        return f"model = huble.sklearn.gradient_boosting_reg(parameters={parameters})"

    def __svm_svr(self, params):
        parameters = {
            "C" : params['C'], 
            "kernel" : params['kernel'],
            "max_iter": params["max_iter"],
            "tol": params["tol"],
        }
        return f"model = huble.sklearn.svm_svr(parameters={parameters})"

    #Clustering

    def __mean_shift(self, params):
        parameters = {
            "bandwidth" : params['bandwidth'],
            "bin_seeding" : params['bin_seeding'],
            "min_bin_freq" : params['min_bin_freq'],
            "cluster_all" : params['cluster_all'],
            "max_iter" : params['max_iter'],
        }
        return f"model = huble.sklearn.mean_shift(parameters={parameters})"


    def __kmeans(self, params):
        parameters = {
            "n_clusters" : params['n_clusters'],
            "init" : params['init'],
            "copy_X" : params['copy_X'],
            "tol" : params['tol'],
            "max_iter" : params['max_iter'],     
            "algorithm" : params['algorithm'],
            "random_state" : params['random_state'],
        }
        return f"model = huble.sklearn.kmeans(parameters={parameters})"

    def __agglomerative(self, params):
        parameters = {
            "n_clusters" : params['n_clusters'],
            "linkage" : params['linkage'],
        }
        return f"model = huble.sklearn.agglomerative(parameters={parameters})"

    def __birch(self, params):
        parameters = {
            "threshold" : params['threshold'],    
            "branching_factor" : params['branching_factor'],
            "n_clusters" : params['n_clusters'],
        }
        return f"model = huble.sklearn.birch(parameters={parameters})"

    def __spectral(self, params):
        parameters = {
            "n_clusters" : params['n_clusters'], 
            "eigen_solver": params["eigen_solver"],
            "n_init": params["n_init"],
            "assign_labels" : params['assign_labels'],
        }
        return f"model = huble.sklearn.spectral(parameters={parameters})"

    def __affinity_propogation(self, params):
        parameters = {
            "damping" : params['damping'],  
            "max_iter" : params['max_iter'],  
            "convergence_iter" : params['convergence_iter'],  
            "affinity" : params['affinity'],  
        }
        return f"model = huble.sklearn.affinity_propogation(parameters={parameters})"

    def __optics(self, params):
        parameters = {
            "min_samples" : params['min_samples'],
            "algorithm" : params['algorithm'],
        }
        return f"model = huble.sklearn.optics(parameters={parameters})"

    def __dbscan(self, params):
        parameters = {
            "eps" : params['eps'],
            "min_samples" : params['min_samples'],
            "algorithm" : params['algorithm'],
        }
        return f"model = huble.sklearn.dbscan(parameters={parameters})"





import lightgbm as lgb
import xgboost as xgb
import catboost as cb
from sklearn import ensemble, linear_model, svm, naive_bayes, neighbors, tree, cluster, kernel_ridge
from ...error.decorators import function_error_handling


@function_error_handling("logistic_regression")
def logistic_regression(**params):
    model = linear_model.LogisticRegression(**params["parameters"])
    return model

@function_error_handling("svm_svc")
def svm_svc(**params):
    model = svm.SVC(**params["parameters"])
    return model

@function_error_handling("gaussian_naive_bayes")
def gaussian_naive_bayes(**params):
    model = naive_bayes.GaussianNB(**params["parameters"])
    return model

@function_error_handling("multinomial_naive_bayes")
def multinomial_naive_bayes(**params):
    model = naive_bayes.MultinomialNB(**params["parameters"])
    return model

@function_error_handling("st_gradient_descent_classifier")
def st_gradient_descent_classifier(**params):
    model = linear_model.SGDClassifier(**params["parameters"])
    return model

@function_error_handling("knn")
def knn(**params):
    model = neighbors.KNeighborsClassifier(**params["parameters"])
    return model

@function_error_handling("decision_tree")
def decision_tree(**params):
    model = tree.DecisionTreeClassifier(**params["parameters"])
    return model

@function_error_handling("random_forest")
def random_forest(**params):
    model = ensemble.RandomForestClassifier(**params["parameters"])
    return model

@function_error_handling("gradient_boosting")
def gradient_boosting(**params):
    model = ensemble.GradientBoostingClassifier(**params["parameters"])
    return model

@function_error_handling("lgbm")
def lgbm(**params):
    model = lgb.LGBMClassifier(**params["parameters"])
    return model

@function_error_handling("xgboost")
def xgboost(**params):
    model = xgb.XGBClassifier(**params["parameters"])
    return model

#Regression

@function_error_handling("linear_regression")
def linear_regression(**params):
    model = linear_model.LinearRegression(**params["parameters"])
    return model

@function_error_handling("lgbm_reg")
def lgbm_reg(**params):
    model = lgb.LGBMRegressor(**params["parameters"])
    return model

@function_error_handling("xgboost_reg")
def xgboost_reg(**params):
    model = xgb.XGBRegressor(**params["parameters"])
    return model

@function_error_handling("catboost_reg")
def catboost_reg(**params):
    model = cb.CatBoostRegressor(**params["parameters"])
    return model

@function_error_handling("st_gradient_descent_reg")
def st_gradient_descent_reg(**params):
    model = linear_model.SGDRegressor(**params["parameters"])
    return model

@function_error_handling("kernel_ridge_reg")
def kernel_ridge_reg(**params):
    model = kernel_ridge.KernelRidge(**params["parameters"])
    return model

@function_error_handling("elastic_net_reg")
def elastic_net_reg(**params):
    model = linear_model.ElasticNet(**params["parameters"])
    return model

@function_error_handling("bayesian_ridge_reg")
def bayesian_ridge_reg(**params):
    model = linear_model.BayesianRidge(**params["parameters"])
    return model

@function_error_handling("gradient_boosting_reg")
def gradient_boosting_reg(**params):
    model = ensemble.GradientBoostingRegressor(**params["parameters"])
    return model

@function_error_handling("svm_svr")
def svm_svr(**params):
    model = svm.SVR(**params["parameters"])
    return model

#Clustering

@function_error_handling("mean_shift")
def mean_shift(**params):
    model = cluster.MeanShift(**params["parameters"])
    return model

@function_error_handling("kmeans")
def kmeans(**params):
    model = cluster.KMeans(**params["parameters"])
    return model

@function_error_handling("agglomerative")
def agglomerative(**params):
    model = cluster.AgglomerativeClustering(**params["parameters"])
    return model

@function_error_handling("birch")
def birch(**params):
    model = cluster.Birch(**params["parameters"])
    return model

@function_error_handling("spectral")
def spectral(**params):
    model = cluster.SpectralClustering(**params["parameters"])
    return model

@function_error_handling("affinity_propogation")
def affinity_propogation(**params):
    model = cluster.AffinityPropagation(**params["parameters"])
    return model

@function_error_handling("optics")
def optics(**params):
    model = cluster.OPTICS(**params["parameters"])
    return model

@function_error_handling("dbscan")
def dbscan(**params):
    model = cluster.DBSCAN(**params["parameters"])
    return model




import huble
model = huble.sklearn.random_forest(parameters={'criterion': 'gini', 'n_estimators': 100, 'max_depth': 'None', 'max_leaf_nodes': 'None', 'random_state': 'None'})
data=huble.util.load_dataset('https://ipfs.filebase.io/ipfs/QmRspeqXi9J2PVTmXYwMaBif9dYWVkNhM8EFomUAfajnT1')
data = huble.sklearn.drop_duplicates(data=data,parameters={})
data = huble.sklearn.remove_mismatch_data(data=data, parameters={'exceptions': []})
print(data.head())
data = huble.sklearn.remove_outliers(data=data,columns=['Ticket'])
data = huble.sklearn.drop_rows_columns(data=data,parameters={'labels': ['PassengerId', 'Pclass', 'Name'], 'axis': 0, 'inplace': False, 'errors': 'raise'})
data = huble.sklearn.clean_data(data=data)
training_dataset, test_dataset = huble.sklearn.train_test_split(data=data,parameters={'test_size': '0.25'})
Model = huble.sklearn.train_model(data=training_dataset, model=model, column='Survived')
metrics = huble.util.evaluate_model(model=Model, test_dataset=test_dataset, target_column= 'Survived', task_type='classification' )



import requests
from pydantic import BaseModel

class Experiment(BaseModel):
    graph: dict
    task_type: str
    target_column: str
    experiment_id:str

    def __init__(self, experiment_id: str) -> None:
        self.experiment_id = experiment_id
        self.graph, self.task_type, self.target_column = self.__get_experiment_details(experiment_id=experiment_id)

    def __get_experiment_details(self, experiment_id: str):
        try:
            response = requests.get(
                f"http://localhost:8000/experiments/{experiment_id}",
            )
            response = response.json()
        except:
            raise Exception("Unable to connect to the server")
        try:
            graph = response["pipelineJSON"]
            project = response["project"]
            target_column = project["targetColumn"]
            task_type = project["taskType"]
        except:
            raise Exception("Invalid Experiment ID")
        return graph, task_type, target_column

    def upload_metrics(self, metrics):
        requests.put(f"http://localhost:8000/experiments/results/{self.experiment_id}", data=metrics)







from huble.error.exceptions import FunctionRuntimeException


def function_error_handling(name: str, *args, **kwargs):
  
      """
      Decorator for handling errors while calling functions in Huble.
      """
      def decorator(func):
        try:
          func(*args, **kwargs)
        except Exception as e:
          raise FunctionRuntimeException(f"{name} failed to execute. Error: {e}")
      return decorator